@@ -3,28 +3,25 @@
 
 import re
 import warnings
+from contextlib import contextmanager
 from collections import defaultdict
+import types as py_types
 from llvmlite import ir
 import llvmlite.binding as llvm
 from .targetinfo import TargetInfo
-from .utils import get_version
 from .errors import UnsupportedError
-from . import libfuncs
+from .utils import get_version
+from . import libfuncs, structure_type
+from rbc import externals
+from rbc.omnisci_backend import mathimpl
+from numba.core import codegen, cpu, compiler_lock, \
+    registry, typing, compiler, sigutils, cgutils, \
+    extending
+from numba.core import errors as nb_errors
 
-if get_version('numba') >= (0, 49):
-    from numba.core import codegen, cpu, compiler_lock, \
-        registry, typing, compiler, sigutils, cgutils, \
-        extending
-    from numba.core import types as nb_types
-    from numba.core import errors as nb_errors
-else:
-    from numba.targets import codegen, cpu, registry
-    from numba import compiler_lock, typing, compiler, \
-        sigutils, cgutils, extending
-    from numba import types as nb_types
-    from numba import errors as nb_errors
 
 int32_t = ir.IntType(32)
+int1_t = ir.IntType(1)
 
 
 def get_called_functions(library, funcname=None):
@@ -69,22 +66,39 @@ def get_called_functions(library, funcname=None):
 
 
 # ---------------------------------------------------------------------------
-# CPU Context classes
-class JITRemoteCPUCodegen(codegen.JITCPUCodegen):
-    # TODO: introduce JITRemoteCodeLibrary?
-    _library_class = codegen.JITCodeLibrary
 
-    def __init__(self, name, target_info):
-        self.target_info = target_info
-        super(JITRemoteCPUCodegen, self).__init__(name)
+class JITRemoteCodeLibrary(codegen.JITCodeLibrary):
+    """JITRemoteCodeLibrary was introduce to prevent numba from calling functions
+    that checks if the module is final. See xnd-project/rbc issue #87.
+    """
+
+    def get_pointer_to_function(self, name):
+        """We can return any random number here! This is just to prevent numba from
+        trying to check if the symbol given by "name" is defined in the module.
+        In cases were RBC is calling an external function (i.e. allocate_varlen_buffer)
+        the symbol will not be defined in the module, resulting in an error.
+        """
+        return 0
+
+    def _finalize_specific(self):
+        """Same as codegen.JITCodeLibrary._finalize_specific but without
+        calling _ensure_finalize at the end
+        """
+        self._codegen._scan_and_fix_unresolved_refs(self._final_module)
+
+
+class JITRemoteCodegen(codegen.JITCPUCodegen):
+    _library_class = JITRemoteCodeLibrary
 
     def _get_host_cpu_name(self):
-        return self.target_info.device_name
+        target_info = TargetInfo()
+        return target_info.device_name
 
     def _get_host_cpu_features(self):
-        features = self.target_info.device_features
-        server_llvm_version = self.target_info.llvm_version
-        if server_llvm_version is None:
+        target_info = TargetInfo()
+        features = target_info.device_features
+        server_llvm_version = target_info.llvm_version
+        if server_llvm_version is None or target_info.is_gpu:
             return ''
         client_llvm_version = llvm.llvm_version_info
 
@@ -95,6 +109,7 @@ class JITRemoteCPUCodegen(codegen.JITCPUCodegen):
                       'serialize', 'amx-int8', 'avx512vp2intersect', 'tsxldtrk',
                       'amx-tile', 'amx-bf16', 'serialize', 'amx-int8',
                       'avx512vp2intersect', 'cx8', 'enqcmd', 'avx512bf16'],
+            (11, 10): ['tsxldtrk', 'amx-tile', 'amx-bf16', 'serialize', 'amx-int8'],
             (9, 8): ['cx8', 'enqcmd', 'avx512bf16'],
         }.get((server_llvm_version[0], client_llvm_version[0]), [])
         for f in remove_features:
@@ -102,86 +117,79 @@ class JITRemoteCPUCodegen(codegen.JITCPUCodegen):
         return features
 
     def _customize_tm_options(self, options):
-        super(JITRemoteCPUCodegen, self)._customize_tm_options(options)
+        super()._customize_tm_options(options)
         # fix reloc_model as the base method sets it using local target
-        if self.target_info.arch.startswith('x86'):
+        target_info = TargetInfo()
+        if target_info.arch.startswith('x86'):
             reloc_model = 'static'
         else:
             reloc_model = 'default'
         options['reloc'] = reloc_model
 
-
-class RemoteCPUContext(cpu.CPUContext):
-
-    def __init__(self, typing_context, target_info):
-        self.target_info = target_info
-        super(RemoteCPUContext, self).__init__(typing_context)
-
-    @compiler_lock.global_compiler_lock
-    def init(self):
-        self.address_size = self.target_info.bits
-        self.is32bit = (self.address_size == 32)
-        self._internal_codegen = JITRemoteCPUCodegen("numba.exec",
-                                                     self.target_info)
-
-        # Map external C functions.
-        # nb.core.externals.c_math_functions.install(self)
-        # TODO, seems problematic only for 32bit cases
-
-        # Initialize NRT runtime
-        # nb.core.argets.cpu.rtsys.initialize(self)
-        # TODO: is this needed for LLVM IR generation?
-
-        # import numba.unicode  # unicode support is not relevant here
-
-    # TODO: overwrite load_additional_registries, call_conv?, post_lowering
-
-# ---------------------------------------------------------------------------
-# GPU Context classes
+    def set_env(self, env_name, env):
+        return None
 
 
-class RemoteGPUTargetContext(cpu.CPUContext):
-
-    def __init__(self, typing_context, target_info):
-        self.target_info = target_info
-        super().__init__(typing_context)
-
-    @compiler_lock.global_compiler_lock
-    def init(self):
-        self.address_size = self.target_info.bits
-        self.is32bit = (self.address_size == 32)
-        self._internal_codegen = JITRemoteCPUCodegen("numba.exec",
-                                                     self.target_info)
-
+class JITRemoteTypingContext(typing.Context):
     def load_additional_registries(self):
-        # libdevice and math from cuda have precedence over the ones from CPU
-        if get_version('numba') >= (0, 52):
-            from numba.cuda import libdeviceimpl, mathimpl
-            from .omnisci_backend import cuda_npyimpl
-            self.install_registry(libdeviceimpl.registry)
-            self.install_registry(mathimpl.registry)
-            self.install_registry(cuda_npyimpl.registry)
-        else:
-            import warnings
-            warnings.warn("libdevice bindings requires Numba 0.52 or newer,"
-                          f" got Numba v{'.'.join(map(str, get_version('numba')))}")
+        for module in externals.__dict__.values():
+            if not isinstance(module, py_types.ModuleType):
+                continue
+
+            typing_registry = getattr(module, 'typing_registry', None)
+            if typing_registry:
+                self.install_registry(typing_registry)
+
+        self.install_registry(mathimpl.typing_registry)
+        self.install_registry(structure_type.typing_registry)
         super().load_additional_registries()
 
 
-class RemoteGPUTypingContext(typing.Context):
+class JITRemoteTargetContext(cpu.CPUContext):
+
+    @compiler_lock.global_compiler_lock
+    def init(self):
+        target_info = TargetInfo()
+        self.address_size = target_info.bits
+        self.is32bit = (self.address_size == 32)
+        self._internal_codegen = JITRemoteCodegen("numba.exec")
+
     def load_additional_registries(self):
-        if get_version('numba') >= (0, 52):
-            from numba.core.typing import npydecl
-            from numba.cuda import cudamath, libdevicedecl
-            self.install_registry(npydecl.registry)
-            self.install_registry(cudamath.registry)
-            self.install_registry(libdevicedecl.registry)
+        for module in externals.__dict__.values():
+            if not isinstance(module, py_types.ModuleType):
+                continue
+
+            if 'rbc.externals' not in module.__name__:
+                continue
+
+            lowering_registry = getattr(module, 'lowering_registry', None)
+            if lowering_registry:
+                self.install_registry(lowering_registry)
+
+        self.install_registry(mathimpl.lowering_registry)
+        self.install_registry(structure_type.lowering_registry)
         super().load_additional_registries()
 
+    def get_executable(self, library, fndesc, env):
+        return None
+
+    def post_lowering(self, mod, library):
+        pass
+
+
 # ---------------------------------------------------------------------------
 # Code generation methods
 
 
+@contextmanager
+def replace_numba_internals_hack():
+    # Hackish solution to prevent numba from calling _ensure_finalize. See issue #87
+    _internal_codegen_bkp = registry.cpu_target.target_context._internal_codegen
+    registry.cpu_target.target_context._internal_codegen = JITRemoteCodegen("numba.exec")
+    yield
+    registry.cpu_target.target_context._internal_codegen = _internal_codegen_bkp
+
+
 def make_wrapper(fname, atypes, rtype, cres, target: TargetInfo, verbose=False):
     """Make wrapper function to numba compile result.
 
@@ -224,10 +232,10 @@ def make_wrapper(fname, atypes, rtype, cres, target: TargetInfo, verbose=False):
     if return_as_first_argument:
         wrapty = ir.FunctionType(ir.VoidType(),
                                  [ll_return_type] + ll_argtypes)
-        wrapfn = module.add_function(wrapty, fname)
+        wrapfn = ir.Function(module, wrapty, fname)
         builder = ir.IRBuilder(wrapfn.append_basic_block('entry'))
         fnty = context.call_conv.get_function_type(rtype, atypes)
-        fn = builder.module.add_function(fnty, cres.fndesc.llvm_func_name)
+        fn = ir.Function(builder.module, fnty, cres.fndesc.llvm_func_name)
         status, out = context.call_conv.call_function(
             builder, fn, rtype, atypes, wrapfn.args[1:])
         with cgutils.if_unlikely(builder, status.is_error):
@@ -235,16 +243,16 @@ def make_wrapper(fname, atypes, rtype, cres, target: TargetInfo, verbose=False):
                 cgutils.printf(builder,
                                f"rbc: {fname} failed with status code %i\n",
                                status.code)
-                cg_fflush(builder)
+                externals.stdio.cg_fflush(builder)
             builder.ret_void()
         builder.store(builder.load(out), wrapfn.args[0])
         builder.ret_void()
     else:
         wrapty = ir.FunctionType(ll_return_type, ll_argtypes)
-        wrapfn = module.add_function(wrapty, fname)
+        wrapfn = ir.Function(module, wrapty, fname)
         builder = ir.IRBuilder(wrapfn.append_basic_block('entry'))
         fnty = context.call_conv.get_function_type(rtype, atypes)
-        fn = builder.module.add_function(fnty, cres.fndesc.llvm_func_name)
+        fn = ir.Function(builder.module, fnty, cres.fndesc.llvm_func_name)
         status, out = context.call_conv.call_function(
             builder, fn, rtype, atypes, wrapfn.args)
         if verbose and target.is_cpu:
@@ -252,7 +260,7 @@ def make_wrapper(fname, atypes, rtype, cres, target: TargetInfo, verbose=False):
                 cgutils.printf(builder,
                                f"rbc: {fname} failed with status code %i\n",
                                status.code)
-                cg_fflush(builder)
+                externals.stdio.cg_fflush(builder)
         builder.ret(out)
 
     cres.library.add_ir_module(module)
@@ -269,9 +277,13 @@ def compile_instance(func, sig,
     succesful.
     """
     flags = compiler.Flags()
-    flags.set('no_compile')
-    flags.set('no_cpython_wrapper')
-    if get_version('numba') >= (0, 49):
+    if get_version('numba') >= (0, 54):
+        flags.no_compile = True
+        flags.no_cpython_wrapper = True
+        flags.no_cfunc_wrapper = True
+    else:
+        flags.set('no_compile')
+        flags.set('no_cpython_wrapper')
         flags.set('no_cfunc_wrapper')
 
     fname = func.__name__ + sig.mangling()
@@ -334,9 +346,20 @@ def compile_instance(func, sig,
     return fname
 
 
+def add_byval_metadata(main_library):
+    module = ir.Module()
+    flag_name = "pass_column_arguments_by_value"
+    mflags = module.add_named_metadata('llvm.module.flags')
+    override_flag = int32_t(4)
+    flag = module.add_metadata([override_flag, flag_name, int1_t(0)])
+    mflags.add(flag)
+    main_library.add_ir_module(module)
+
+
 def compile_to_LLVM(functions_and_signatures,
-                    target: TargetInfo,
+                    target_info: TargetInfo,
                     pipeline_class=compiler.Compiler,
+                    user_defined_llvm_ir=None,
                     debug=False):
     """Compile functions with given signatures to target specific LLVM IR.
 
@@ -346,6 +369,9 @@ def compile_to_LLVM(functions_and_signatures,
       Specify a list of Python function and its signatures pairs.
     target : TargetInfo
       Specify target device information.
+    user_defined_llvm_ir : {None, str, ModuleRef}
+      Specify user-defined LLVM IR module that is linked in to the
+      returned module.
     debug : bool
 
     Returns
@@ -356,84 +382,79 @@ def compile_to_LLVM(functions_and_signatures,
     """
     target_desc = registry.cpu_target
 
-    if target is None:
-        # RemoteJIT
-        target = TargetInfo.host()
-        typing_context = target_desc.typing_context
-        target_context = target_desc.target_context
-    else:
-        # OmnisciDB target
-        if target.is_cpu:
-            typing_context = typing.Context()
-            target_context = RemoteCPUContext(typing_context, target)
-        elif target.is_gpu:
-            typing_context = RemoteGPUTypingContext()
-            target_context = RemoteGPUTargetContext(typing_context, target)
-        else:
-            raise ValueError(f'Unknown target {target.name}')
+    typing_context = JITRemoteTypingContext()
+    target_context = JITRemoteTargetContext(typing_context)
 
-        # Bring over Array overloads (a hack):
-        target_context._defns = target_desc.target_context._defns
+    # Bring over Array overloads (a hack):
+    target_context._defns = target_desc.target_context._defns
 
-    codegen = target_context.codegen()
-    main_library = codegen.create_library('rbc.irtools.compile_to_IR')
-    main_module = main_library._final_module
+    with replace_numba_internals_hack():
+        codegen = target_context.codegen()
+        main_library = codegen.create_library('rbc.irtools.compile_to_IR')
+        main_module = main_library._final_module
 
-    succesful_fids = []
-    function_names = []
-    for func, signatures in functions_and_signatures:
-        for fid, sig in signatures.items():
-            fname = compile_instance(func, sig, target, typing_context,
-                                     target_context, pipeline_class,
-                                     main_library,
-                                     debug=debug)
-            if fname is not None:
-                succesful_fids.append(fid)
-                function_names.append(fname)
+        if user_defined_llvm_ir is not None:
+            if isinstance(user_defined_llvm_ir, str):
+                user_defined_llvm_ir = llvm.parse_assembly(user_defined_llvm_ir)
+            assert isinstance(user_defined_llvm_ir, llvm.ModuleRef)
+            main_module.link_in(user_defined_llvm_ir, preserve=True)
 
-    main_library._optimize_final_module()
+        succesful_fids = []
+        function_names = []
+        for func, signatures in functions_and_signatures:
+            for fid, sig in signatures.items():
+                fname = compile_instance(func, sig, target_info, typing_context,
+                                         target_context, pipeline_class,
+                                         main_library,
+                                         debug=debug)
+                if fname is not None:
+                    succesful_fids.append(fid)
+                    function_names.append(fname)
 
-    # Remove unused defined functions and declarations
-    used_symbols = defaultdict(set)
-    for fname in function_names:
-        for k, v in get_called_functions(main_library, fname).items():
-            used_symbols[k].update(v)
-
-    all_symbols = get_called_functions(main_library)
-
-    unused_symbols = defaultdict(set)
-    for k, lst in all_symbols.items():
-        if k == 'libraries':
-            continue
-        for fn in lst:
-            if fn not in used_symbols[k]:
-                unused_symbols[k].add(fn)
-
-    changed = False
-    for f in main_module.functions:
-        fn = f.name
-        if fn.startswith('llvm.'):
-            if f.name in unused_symbols['intrinsics']:
-                f.linkage = llvm.Linkage.external
-                changed = True
-        elif f.is_declaration:
-            if f.name in unused_symbols['declarations']:
-                f.linkage = llvm.Linkage.external
-                changed = True
-        else:
-            if f.name in unused_symbols['defined']:
-                f.linkage = llvm.Linkage.private
-                changed = True
-
-    # TODO: determine unused global_variables and struct_types
-
-    if changed:
+        add_byval_metadata(main_library)
         main_library._optimize_final_module()
 
-    main_module.verify()
-    main_library._finalized = True
-    main_module.triple = target.triple
-    main_module.data_layout = target.datalayout
+        # Remove unused defined functions and declarations
+        used_symbols = defaultdict(set)
+        for fname in function_names:
+            for k, v in get_called_functions(main_library, fname).items():
+                used_symbols[k].update(v)
+
+        all_symbols = get_called_functions(main_library)
+
+        unused_symbols = defaultdict(set)
+        for k, lst in all_symbols.items():
+            if k == 'libraries':
+                continue
+            for fn in lst:
+                if fn not in used_symbols[k]:
+                    unused_symbols[k].add(fn)
+
+        changed = False
+        for f in main_module.functions:
+            fn = f.name
+            if fn.startswith('llvm.'):
+                if f.name in unused_symbols['intrinsics']:
+                    f.linkage = llvm.Linkage.external
+                    changed = True
+            elif f.is_declaration:
+                if f.name in unused_symbols['declarations']:
+                    f.linkage = llvm.Linkage.external
+                    changed = True
+            else:
+                if f.name in unused_symbols['defined']:
+                    f.linkage = llvm.Linkage.private
+                    changed = True
+
+        # TODO: determine unused global_variables and struct_types
+
+        if changed:
+            main_library._optimize_final_module()
+
+        main_module.verify()
+        main_library._finalized = True
+        main_module.triple = target_info.triple
+        main_module.data_layout = target_info.datalayout
 
     return main_module, succesful_fids
 
@@ -464,6 +485,10 @@ def compile_IR(ir):
         ir, re.M).group('triple')
 
     # Create execution engine
+    llvm.initialize()
+    llvm.initialize_all_targets()
+    llvm.initialize_all_asmprinters()
+
     target = llvm.Target.from_triple(triple)
     target_machine = target.create_target_machine()
     backing_mod = llvm.parse_assembly("")
@@ -479,48 +504,27 @@ def compile_IR(ir):
     return engine
 
 
-def cg_fflush(builder):
-    int8_t = ir.IntType(8)
-    fflush_fnty = ir.FunctionType(int32_t, [int8_t.as_pointer()])
-    fflush_fn = builder.module.get_or_insert_function(
-        fflush_fnty, name='fflush')
-
-    builder.call(fflush_fn, [int8_t.as_pointer()(None)])
+def IS_CPU():
+    pass
 
 
-@extending.intrinsic
-def fflush(typingctx):
-    """fflush that can be called from Numba jit-decorated functions.
-
-    Note: fflush is available only for CPU target.
-    """
-    sig = nb_types.void(nb_types.void)
-
-    def codegen(context, builder, signature, args):
-        target_info = TargetInfo()
-        if target_info.is_cpu:
-            cg_fflush(builder)
-
-    return sig, codegen
-
-
-@extending.intrinsic
-def printf(typingctx, format_type, *args):
-    """printf that can be called from Numba jit-decorated functions.
-
-    Note: printf is available only for CPU target.
-    """
-
-    if isinstance(format_type, nb_types.StringLiteral):
-        sig = nb_types.void(format_type, nb_types.BaseTuple.from_types(args))
-
-        def codegen(context, builder, signature, args):
-            target_info = TargetInfo()
-            if target_info.is_cpu:
-                cgutils.printf(builder, format_type.literal_value, *args[1:])
-                cg_fflush(builder)
-
-        return sig, codegen
-
+@extending.overload(IS_CPU, inline="always")
+def is_cpu_impl():
+    target_info = TargetInfo()
+    if target_info.is_cpu:
+        return lambda: True
     else:
-        raise TypeError(f'expected StringLiteral but got {type(format_type).__name__}')
+        return lambda: False
+
+
+def IS_GPU():
+    pass
+
+
+@extending.overload(IS_GPU, inline="always")
+def is_gpu_impl():
+    target_info = TargetInfo()
+    if target_info.is_gpu:
+        return lambda: True
+    else:
+        return lambda: False
