@@ -1,8 +1,11 @@
 from .numpy_funcs import *  # noqa: F401, F403
-from .npy_mathimpl import *  # noqa: F401, F403
+from .npyimpl import *  # noqa: F401, F403
 from .omnisci_array import *  # noqa: F401, F403
 from .omnisci_column import *  # noqa: F401, F403
 from .omnisci_bytes import *  # noqa: F401, F403
+from .omnisci_metatype import *  # noqa: F401, F403
+from .omnisci_text_encoding import *  # noqa: F401, F403
 from .numpy_ufuncs import *  # noqa: F401, F403
 from .omnisci_pipeline import *  # noqa: F401, F403
 from .python_operators import *  # noqa: F401, F403
+from .column_list import *  # noqa: F401, F403
