@@ -82,14 +82,26 @@ def get_literal_return(func, verbose=False):
                 print(source)
 
 
+def _global_omnisci():
+    """Implements singleton of a global RemoteOmnisci instance.
+    """
+    config = get_client_config()
+    omnisci = RemoteOmnisci(**config)
+    while True:
+        yield omnisci
+
+
+global_omnisci_singleton = _global_omnisci()  # generator object
+
+
 def is_available(_cache={}):
     """Return version tuple and None if OmnisciDB server is accessible or
     recent enough. Otherwise return None and the reason about
     unavailability.
     """
+
     if not _cache:
-        config = get_client_config()
-        omnisci = RemoteOmnisci(**config)
+        omnisci = next(global_omnisci_singleton)
         try:
             version = omnisci.version
         except Exception as msg:
@@ -281,7 +293,7 @@ class RemoteOmnisci(RemoteJIT):
     def version(self):
         if self._version is None:
             version = self.thrift_call('get_version')
-            return parse_version(version)
+            self._version = parse_version(version)
         return self._version
 
     @property
@@ -378,6 +390,9 @@ class RemoteOmnisci(RemoteJIT):
           The table must have the specified columns defined.
 
         """
+        if not self._null_values:
+            self.retrieve_targets()  # initializes null values
+
         int_col_types = ['TINYINT', 'SMALLINT', 'INT', 'BIGINT', 'BOOL',
                          'DECIMAL', 'TIME', 'TIMESTAMP', 'DATE']
         real_col_types = ['FLOAT', 'DOUBLE']
@@ -563,13 +578,18 @@ class RemoteOmnisci(RemoteJIT):
             for i in range(nrows):
                 yield tuple(columns[j][i] for j in range(ncols))
 
+    def query_requires_register(self, query):
+        """Check if given query requires registration step.
+        """
+        names = '|'.join(self.get_pending_names())
+        return names and re.search(r'\b(' + names + r')\b', query, re.I) is not None
+
     def sql_execute(self, query):
         """Execute SQL query in OmnisciDB server.
 
         Parameters
         ----------
         query : str
-
           SQL query string containing exactly one query. Multiple
           queries are not supported.
 
@@ -579,8 +599,10 @@ class RemoteOmnisci(RemoteJIT):
           Row description object
         results : iterator
           Iterator over rows.
+
         """
-        self.register()
+        if self.query_requires_register(query):
+            self.register()
         columnar = True
         if self.debug:
             print('  %s;' % (query))
