@@ -1,4 +1,4 @@
-__all__ = ['omnisci_fixture']
+__all__ = ['omnisci_fixture', 'sql_execute']
 
 
 import os
@@ -7,6 +7,16 @@ import warnings
 from collections import defaultdict
 
 
+def sql_execute(query):
+    """Execute a SQL statement to omniscidb server using global instance.
+
+    Use when the query does not require registration of new UDF/UDTFs.
+    """
+    rbc_omnisci = pytest.importorskip('rbc.omniscidb')
+    omnisci = next(rbc_omnisci.global_omnisci_singleton)
+    return omnisci.sql_execute(query)
+
+
 def omnisci_fixture(caller_globals, minimal_version=(0, 0),
                     suffices=['', '10', 'null', 'array', 'arraynull'],
                     load_columnar=True, debug=False):
