@@ -60,10 +60,12 @@ def nb_version():
 
 @pytest.fixture(scope='module')
 def omnisci():
+    # TODO: use omnisci_fixture from rbc/tests/__init__.py
     config = rbc_omnisci.get_client_config(debug=not True)
     m = rbc_omnisci.RemoteOmnisci(**config)
     table_name = 'rbc_test_omnisci'
-    m.sql_execute('DROP TABLE IF EXISTS {table_name}'.format(**locals()))
+
+    m.sql_execute(f'DROP TABLE IF EXISTS {table_name}')
     sqltypes = ['FLOAT', 'DOUBLE', 'TINYINT', 'SMALLINT', 'INT', 'BIGINT',
                 'BOOLEAN']
     # todo: TEXT ENCODING DICT, TEXT ENCODING NONE, TIMESTAMP, TIME,
@@ -73,9 +75,7 @@ def omnisci():
     colnames = ['f4', 'f8', 'i1', 'i2', 'i4', 'i8', 'b']
     table_defn = ',\n'.join('%s %s' % (n, t)
                             for t, n in zip(sqltypes, colnames))
-    m.sql_execute(
-        'CREATE TABLE IF NOT EXISTS {table_name} ({table_defn});'
-        .format(**locals()))
+    m.sql_execute(f'CREATE TABLE IF NOT EXISTS {table_name} ({table_defn});')
 
     def row_value(row, col, colname):
         if colname == 'b':
@@ -86,11 +86,10 @@ def omnisci():
     for i in range(rows):
         table_row = ', '.join(str(row_value(i, j, n))
                               for j, n in enumerate(colnames))
-        m.sql_execute(
-            'INSERT INTO {table_name} VALUES ({table_row})'.format(**locals()))
+        m.sql_execute(f'INSERT INTO {table_name} VALUES ({table_row})')
     m.table_name = table_name
     yield m
-    m.sql_execute('DROP TABLE IF EXISTS {table_name}'.format(**locals()))
+    m.sql_execute(f'DROP TABLE IF EXISTS {table_name}')
 
 
 def test_redefine(omnisci):
@@ -101,7 +100,7 @@ def test_redefine(omnisci):
         return x + 1
 
     desrc, result = omnisci.sql_execute(
-        'select i4, incr(i4) from {omnisci.table_name}'.format(**locals()))
+        f'select i4, incr(i4) from {omnisci.table_name}')
     for x, x1 in result:
         assert x1 == x + 1
 
@@ -111,7 +110,7 @@ def test_redefine(omnisci):
         return x + 2
 
     desrc, result = omnisci.sql_execute(
-        'select i4, incr(i4) from {omnisci.table_name}'.format(**locals()))
+        f'select i4, incr(i4) from {omnisci.table_name}')
     for x, x1 in result:
         assert x1 == x + 2
 
