@@ -11,10 +11,12 @@ pytestmark = pytest.mark.skipif(not available_version, reason=reason)
 
 @pytest.fixture(scope='module')
 def omnisci():
+    # TODO: use omnisci_fixture from rbc/tests/__init__.py
     config = rbc_omnisci.get_client_config(debug=not True)
     m = rbc_omnisci.RemoteOmnisci(**config)
     table_name = 'rbc_test_omnisci_array'
-    m.sql_execute('DROP TABLE IF EXISTS {table_name}'.format(**locals()))
+
+    m.sql_execute(f'DROP TABLE IF EXISTS {table_name}')
     sqltypes = ['FLOAT[]', 'DOUBLE[]',
                 'TINYINT[]', 'SMALLINT[]', 'INT[]', 'BIGINT[]',
                 'BOOLEAN[]']
@@ -44,12 +46,11 @@ def omnisci():
     for i in range(rows):
         table_row = ', '.join(str(row_value(i, j, n))
                               for j, n in enumerate(colnames))
-        m.sql_execute(
-            'INSERT INTO {table_name} VALUES ({table_row})'.format(**locals()))
+        m.sql_execute(f'INSERT INTO {table_name} VALUES ({table_row})')
     m.table_name = table_name
     yield m
     try:
-        m.sql_execute('DROP TABLE IF EXISTS {table_name}'.format(**locals()))
+        m.sql_execute(f'DROP TABLE IF EXISTS {table_name}')
     except Exception as msg:
         print('%s in deardown' % (type(msg)))
 
